﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ServiceTrackerPresenter;
using ServiceTrackerLogic;

namespace ServiceTracker
{
	public partial class MainForm : Form, IAccountView
	{
		protected AccountPresenter presenter;

		public MainForm()
		{
			InitializeComponent();
			presenter = new AccountPresenter(this);
		}

		#region IAccountView Members

		public event AccountViewEventDelegate SearchCustomersRequested;
		public event AccountViewEventDelegate AccountViewLoad;
		public event AccountViewEventDelegate AddAccountRequested;
		
		private void MainForm_Load(object sender, EventArgs e)
		{
			if (AccountViewLoad != null)
				AccountViewLoad(this, this);
		}

		private void SubmitSearch_Click(object sender, EventArgs e)
		{
			if (SearchCustomersRequested != null)
				SearchCustomersRequested(this, this);
		}

		public void OpenAccountWindow()
		{
			((AddAccountForm)this.AddAccountForm).ShowDialog();
		}

		public virtual IAddAccountView AddAccountForm
		{
			get { return new AddAccountForm(); }
		}

		public IAccountGrid AccountGrid
		{
			get
			{
				return new AccountsDataGridWrapper(AccountsGrid);
			}
		}

		public string SearchCriteria
		{
			get
			{
				return SearchCriteriaText.Text;
			}
			set
			{
				SearchCriteriaText.Text = value;
			}
		}

		#endregion

		private void AddAccount_Click(object sender, EventArgs e)
		{
			if (AddAccountRequested != null)
				AddAccountRequested(this, this);
		}

	}

	class AccountsDataGridWrapper : IAccountGrid
	{
		DataGridView dgv;

		public AccountsDataGridWrapper(DataGridView dgview)
		{
			dgv = dgview;
		}

		public object DataSource
		{
			get { return dgv.DataSource; }
			set { dgv.DataSource = value; }
		}

		public DataGridView DataGridView
		{
			get { return dgv; }
		}
	}
}
