﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ServiceTrackerLogic;

namespace ServiceTrackerPresenter
{
	public class AccountPresenter
	{
		IAccountView accountView;

		public AccountPresenter(IAccountView view)
		{
			accountView = view;
			accountView.AccountViewLoad += new AccountViewEventDelegate(view_AccountViewLoad);
			accountView.SearchCustomersRequested += new AccountViewEventDelegate(view_SearchCustomersRequested);
			accountView.AddAccountRequested += new AccountViewEventDelegate(view_AddAccountRequested);
			Accounts.AccountStore.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(AccountStore_CollectionChanged);
		}

		void AccountStore_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
		{
			//Must set DataSource to null first or else
			//the GUI won't recognize the change
			accountView.AccountGrid.DataSource = null;
			accountView.AccountGrid.DataSource = Accounts.AccountStore;
		}

		void view_AddAccountRequested(object sender, IAccountView view)
		{
			view.OpenAccountWindow();
		}

		void view_AccountViewLoad(object sender, IAccountView view)
		{
			view.AccountGrid.DataSource = LoadAccounts();
		}

		private Accounts LoadAccounts()
		{
			return Accounts.AccountStore;
		}

		void view_SearchCustomersRequested(object sender, IAccountView view)
		{
			Accounts accounts = LoadAccounts();
			accounts = accounts.FindAccount(view.SearchCriteria);
			view.AccountGrid.DataSource = accounts;
		}
	}
}
