﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ServiceTracker;
using ServiceTrackerPresenter;
using ServiceTrackerLogic;
using System.Windows.Forms;

namespace ServiceTrackerTests
{
	/// <summary>
	/// Summary description for UnitTest1
	/// </summary>
	[TestClass]
	public class MainFormTests
	{
		[TestCleanup]
		public void TearDown()
		{
			//Clean up AccountStore
			Accounts.AccountStore = null;
		}

		[TestMethod]
		public void MainFormCallsAccountViewLoadOnLoad()
		{
			StubMainForm stubForm = new StubMainForm();
			StubAccountPresenter presenter = new StubAccountPresenter(stubForm);
			stubForm.SetPresenter(presenter);
			stubForm.FireLoadEvent();
			Assert.IsTrue(presenter.AccountViewLoadEventFired);
		}

		[TestMethod]
		public void MainFormCallsSearchCustomersRequestedOnClick()
		{
			StubMainForm stubForm = new StubMainForm();
			StubAccountPresenter presenter = new StubAccountPresenter(stubForm);
			stubForm.SetPresenter(presenter);
			stubForm.Show();
			stubForm.FireSearchClickEvent();
			Assert.IsTrue(presenter.SearchCustomersRequestedEventFired);
		}

		[TestMethod]
		public void MainFormCallsAddAccountRequestedOnClick()
		{
			StubMainForm stubForm = new StubMainForm();
			StubAccountPresenter presenter = new StubAccountPresenter(stubForm);
			stubForm.SetPresenter(presenter);
			stubForm.Show();
			stubForm.FireAddAccountClickEvent();
			Assert.IsTrue(presenter.AddCustomerRequestedEventFired);
		}

		[TestMethod]
		public void OpenAccountWindowOpensRightWindowUsingShowDialog()
		{
			StubMainForm stubForm = new StubMainForm();
			StubAccountPresenter presenter = new StubAccountPresenter(stubForm);
			stubForm.SetPresenter(presenter);
			stubForm.Show();
			stubForm.FireAddAccountClickEvent();
			Assert.IsTrue(stubForm.AddAccountFormRequested, "AddAccountFormRequested");
			Assert.IsTrue(stubForm.AddAccountFormOpenedWithShowDialog, "AddAccountFormOpenedWithShowDialog");
		}

		[TestMethod]
		public void AddAccountFormPropertyReturnsCorrectClass()
		{
			MainForm form = new MainForm();
			AddAccountForm addAccountForm = form.AddAccountForm as AddAccountForm;
			Assert.IsNotNull(addAccountForm);
		}

		[TestMethod]
		public void DataGridUpdatedWhenAccountsCollectionChanged()
		{
			StubMainForm view = new StubMainForm();
			StubAccountPresenter presenter = new StubAccountPresenter(view);
			view.FireLoadEvent();
			Accounts.AccountStore.Add(new Account("Faked Out", "813-111-2222"));
			Assert.AreEqual(Accounts.AccountStore, view.AccountGrid.DataSource);
		}

		[TestMethod]
		public void RowsDisplayedUpdatesWhenAccountsCollectionChanged()
		{
			using (StubMainForm view = new StubMainForm())
			{
				view.Show();
				DataGridView realGrid = view.GetAccountsGrid();
				Assert.IsNotNull(realGrid, "realGrid");
				Assert.AreEqual(2, realGrid.RowCount);
				Accounts.AccountStore.Add(new Account("Faked out", "813-111-2222"));
				Assert.AreEqual(3, realGrid.RowCount);
				view.Close();
			}
		}
	}

	class StubAccountPresenter : AccountPresenter
	{
		private IAccountView view;
		public bool AccountViewLoadEventFired = false;
		public bool SearchCustomersRequestedEventFired = false;
		public bool AddCustomerRequestedEventFired = false;

		public StubAccountPresenter(IAccountView view)
			: base(view)
		{
			
			this.view = view;
			view.AccountViewLoad += new AccountViewEventDelegate(view_AccountViewLoad);
			view.SearchCustomersRequested += new AccountViewEventDelegate(view_SearchCustomersRequested);
			view.AddAccountRequested += new AccountViewEventDelegate(view_AddAccountRequested);
		}

		void view_AddAccountRequested(object sender, IAccountView view)
		{
			AddCustomerRequestedEventFired = true;
		}

		void view_SearchCustomersRequested(object sender, IAccountView view)
		{
			SearchCustomersRequestedEventFired = true;
		}

		void view_AccountViewLoad(object sender, IAccountView view)
		{
			AccountViewLoadEventFired = true;
		}
	}

	class StubMainForm : MainForm
	{
		public bool AddAccountFormRequested = false;
		private StubAddAccountForm stubAddAccountForm;

		public DataGridView GetAccountsGrid()
		{
			return this.AccountsGrid;
		}

		public override IAddAccountView AddAccountForm
		{
			get 
			{
				AddAccountFormRequested = true;
				stubAddAccountForm = new StubAddAccountForm();
				return stubAddAccountForm; 
			}
		}

		public bool AddAccountFormOpenedWithShowDialog
		{
			get { return stubAddAccountForm.ShowDialogCalled; }
		}

		public void SetPresenter(AccountPresenter presenter)
		{
			base.presenter = presenter;
		}

		public void FireLoadEvent()
		{
			this.OnLoad(null);
		}

		public void FireSearchClickEvent()
		{
			base.SubmitSearch.PerformClick();
		}

		public void FireAddAccountClickEvent()
		{
			base.AddAccount.PerformClick();
		}
	}

	class StubAddAccountForm : AddAccountForm
	{
		public bool ShowDialogCalled = false;

		public StubAddAccountForm()
			: base()
		{
			this.Shown += new EventHandler(StubAddAccountForm_Shown);
		}

		void StubAddAccountForm_Shown(object sender, EventArgs e)
		{
			ShowDialogCalled = this.Modal;
			this.Close();
		}
	}
}
